      SUBROUTINE DGESL(A,LDA,N,IPVT,B,JOB)                              
      INTEGER LDA,N,IPVT(1),JOB                                         
      DOUBLE PRECISION A(LDA,1),B(1)                                    
C                                                                       
C     dgesl solves the double precision system                          
C     a * x = b  or  trans(a) * x = b                                   
C     using the factors computed by dgeco or dgefa.                     
C                                                                       
C     on entry                                                          
C                                                                       
C        a       double precision(lda, n)                               
C                the output from dgeco or dgefa.                        
C                                                                       
C        lda     integer                                                
C                the leading dimension of the array  a .                
C                                                                       
C        n       integer                                                
C                the order of the matrix  a .                           
C                                                                       
C        ipvt    integer(n)                                             
C                the pivot vector from dgeco or dgefa.                  
C                                                                       
C        b       double precision(n)                                    
C                the right hand side vector.                            
C                                                                       
C        job     integer                                                
C                = 0         to solve  a*x = b ,                        
C                = nonzero   to solve  trans(a)*x = b  where            
C                            trans(a)  is the transpose.                
C                                                                       
C     on return                                                         
C                                                                       
C        b       the solution vector  x .                               
C                                                                       
C     error condition                                                   
C                                                                       
C        a division by zero will occur if the input factor contains a   
C        zero on the diagonal.  technically this indicates singularity  
C        but it is often caused by improper arguments or improper       
C        setting of lda .  it will not occur if the subroutines are     
C        called correctly and if dgeco has set rcond .gt. 0.0           
C        or dgefa has set info .eq. 0 .                                 
C                                                                       
C     to compute  inverse(a) * c  where  c  is a matrix                 
C     with  p  columns                                                  
C           call dgeco(a,lda,n,ipvt,rcond,z)                            
C           if (rcond is too small) go to ...                           
C           do 10 j = 1, p                                              
C              call dgesl(a,lda,n,ipvt,c(1,j),0)                        
C        10 continue                                                    
C                                                                       
C     linpack. this version dated 08/14/78 .                            
C     cleve moler, university of new mexico, argonne national lab.      
C                                                                       
C     subroutines and functions                                         
C                                                                       
C     blas daxpy,ddot                                                   
C                                                                       
C     internal variables                                                
C                                                                       
      DOUBLE PRECISION DDOT,T                                           
      INTEGER K,KB,L,NM1                                                
C                                                                       
      NM1 = N - 1                                                       
      IF (JOB .NE. 0) GO TO 50                                          
C                                                                       
C        job = 0 , solve  a * x = b                                     
C        first solve  l*y = b                                           
C                                                                       
         IF (NM1 .LT. 1) GO TO 30                                       
         DO 20 K = 1, NM1                                               
            L = IPVT(K)                                                 
            T = B(L)                                                    
            IF (L .EQ. K) GO TO 10                                      
               B(L) = B(K)                                              
               B(K) = T                                                 
   10       CONTINUE                                                    
            CALL DAXPY(N-K,T,A(K+1,K),1,B(K+1),1)                       
   20    CONTINUE                                                       
   30    CONTINUE                                                       
C                                                                       
C        now solve  u*x = y                                             
C                                                                       
         DO 40 KB = 1, N                                                
            K = N + 1 - KB                                              
            B(K) = B(K)/A(K,K)                                          
            T = -B(K)                                                   
            CALL DAXPY(K-1,T,A(1,K),1,B(1),1)                           
   40    CONTINUE                                                       
      GO TO 100                                                         
   50 CONTINUE                                                          
C                                                                       
C        job = nonzero, solve  trans(a) * x = b                         
C        first solve  trans(u)*y = b                                    
C                                                                       
         DO 60 K = 1, N                                                 
            T = DDOT(K-1,A(1,K),1,B(1),1)                               
            B(K) = (B(K) - T)/A(K,K)                                    
   60    CONTINUE                                                       
C                                                                       
C        now solve trans(l)*x = y                                       
C                                                                       
         IF (NM1 .LT. 1) GO TO 90                                       
         DO 80 KB = 1, NM1                                              
            K = N - KB                                                  
            B(K) = B(K) + DDOT(N-K,A(K+1,K),1,B(K+1),1)                 
            L = IPVT(K)                                                 
            IF (L .EQ. K) GO TO 70                                      
               T = B(L)                                                 
               B(L) = B(K)                                              
               B(K) = T                                                 
   70       CONTINUE                                                    
   80    CONTINUE                                                       
   90    CONTINUE                                                       
  100 CONTINUE                                                          
      RETURN                                                            
      END                                                               
